@@ -1,14 +1,10 @@
-#define NDPI_LIB_COMPILATION
-
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 #include "fuzz_common_code.h"
 
 #include <stdint.h>
 #include <stdio.h>
 
-extern void processCertificateElements(struct ndpi_detection_module_struct *ndpi_struct,
-				       struct ndpi_flow_struct *flow,
-				       u_int16_t p_offset, u_int16_t certificate_len);
 struct ndpi_tcphdr tcph;
 struct ndpi_iphdr iph;
 struct ndpi_ipv6hdr iphv6;
@@ -21,7 +17,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
   int is_ipv6;
 
   if (ndpi_struct == NULL) {
-    fuzz_init_detection_module(&ndpi_struct);
+    fuzz_init_detection_module(&ndpi_struct, NULL);
     ndpi_flow = ndpi_calloc(1, sizeof(struct ndpi_flow_struct));
   }
 
