@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (C) 2011-24 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -82,13 +82,15 @@ typedef struct default_ports_tree_node {
 
 #define LINE_ENDS(ndpi_int_one_line_struct, string_to_compare) \
   ((ndpi_int_one_line_struct).len >= strlen(string_to_compare) && \
-   memcmp((ndpi_int_one_line_struct).ptr + \
-          ((ndpi_int_one_line_struct).len - strlen(string_to_compare)), \
-          string_to_compare, strlen(string_to_compare)) == 0)
+   ndpi_strncasestr((const char *)((ndpi_int_one_line_struct).ptr) + \
+                    ((ndpi_int_one_line_struct).len - strlen(string_to_compare)), \
+                    string_to_compare, strlen(string_to_compare)) == \
+   (const char *)((ndpi_int_one_line_struct).ptr) + ((ndpi_int_one_line_struct).len - strlen(string_to_compare)))
 
 #define LINE_CMP(ndpi_int_one_line_struct, string_to_compare, string_to_compare_length) \
   ((ndpi_int_one_line_struct).ptr != NULL && \
-   memcmp((ndpi_int_one_line_struct).ptr, string_to_compare, string_to_compare_length) == 0)
+   ndpi_strncasestr((const char *)((ndpi_int_one_line_struct).ptr), string_to_compare, \
+                    string_to_compare_length) == (const char *)((ndpi_int_one_line_struct).ptr))
 
 #define NDPI_MAX_PARSE_LINES_PER_PACKET                         64
 
@@ -102,7 +104,6 @@ struct ndpi_packet_struct {
   const struct ndpi_ipv6hdr *iphv6;
   const struct ndpi_tcphdr *tcp;
   const struct ndpi_udphdr *udp;
-  const u_int8_t *generic_l4_ptr;	/* is set only for non tcp-udp traffic */
   const u_int8_t *payload;
 
   u_int64_t current_time_ms;
@@ -121,8 +122,26 @@ struct ndpi_packet_struct {
   struct ndpi_int_one_line_struct http_origin;
   struct ndpi_int_one_line_struct server_line;
   struct ndpi_int_one_line_struct http_method;
+  struct ndpi_int_one_line_struct upgrade_line;
   struct ndpi_int_one_line_struct http_response; /* the first "word" in this pointer is the
 						    response code in the packet (200, etc) */
+  struct ndpi_int_one_line_struct bootid;
+  struct ndpi_int_one_line_struct usn;
+  struct ndpi_int_one_line_struct cache_controle;
+  struct ndpi_int_one_line_struct location;
+  struct ndpi_int_one_line_struct household_smart_speaker_audio;
+  struct ndpi_int_one_line_struct rincon_household;
+  struct ndpi_int_one_line_struct rincon_bootseq;
+  struct ndpi_int_one_line_struct rincon_wifimode;
+  struct ndpi_int_one_line_struct rincon_variant;
+  struct ndpi_int_one_line_struct sonos_securelocation;
+  struct ndpi_int_one_line_struct securelocation_upnp;
+  struct ndpi_int_one_line_struct location_smart_speaker_audio;
+  struct ndpi_int_one_line_struct nt;
+  struct ndpi_int_one_line_struct nts;
+  struct ndpi_int_one_line_struct man;
+  struct ndpi_int_one_line_struct mx;
+  struct ndpi_int_one_line_struct st;
 
   u_int16_t l3_packet_len;
   u_int16_t payload_packet_len;
@@ -171,6 +190,9 @@ struct ndpi_global_context {
   
   /* FPC DNS cache */
   struct ndpi_lru_cache *fpc_dns_global_cache;
+
+  /* Signal STUN cache */
+  struct ndpi_lru_cache *signal_global_cache;
 };
 
 #define CFG_MAX_LEN	256
@@ -198,7 +220,12 @@ struct ndpi_detection_module_config_struct {
   int libgcrypt_init;
   int guess_on_giveup;
   int compute_entropy;
+  int address_cache_size;
   int fpc_enabled;
+  int guess_ip_before_port;
+  int use_client_ip_in_guess;
+  int use_client_port_in_guess;
+  int tcp_fingerprint_enabled;
   
   char filename_config[CFG_MAX_LEN];
 
@@ -227,15 +254,37 @@ struct ndpi_detection_module_config_struct {
   int fpc_dns_cache_num_entries;
   int fpc_dns_cache_ttl;
   int fpc_dns_cache_scope;
-  
+  int signal_cache_num_entries;
+  int signal_cache_ttl;
+  int signal_cache_scope;
+
   /* Protocols */
 
+  int http_request_content_type_enabled;
+  int http_referer_enabled;
+  int http_host_enabled;
+  int http_username_enabled;
+  int http_password_enabled;
+
   int tls_certificate_expire_in_x_days;
   int tls_app_blocks_tracking_enabled;
+  int tls_heuristics;
+  int tls_heuristics_max_packets;
+  int tls_versions_supported_enabled;
+  int tls_alpn_negotiated_enabled;
+  int tls_cipher_enabled;
   int tls_sha1_fingerprint_enabled;
-  int tls_ja3c_fingerprint_enabled;
+  int tls_cert_server_names_enabled;
+  int tls_cert_validity_enabled;
+  int tls_cert_issuer_enabled;
+  int tls_cert_subject_enabled;
+  int tls_broswer_enabled;
   int tls_ja3s_fingerprint_enabled;
   int tls_ja4c_fingerprint_enabled;
+  int tls_ja4r_fingerprint_enabled;
+  int tls_subclassification_enabled;
+
+  int quic_subclassification_enabled;
 
   int smtp_opportunistic_tls_enabled;
 
@@ -245,18 +294,29 @@ struct ndpi_detection_module_config_struct {
 
   int ftp_opportunistic_tls_enabled;
 
+  int sip_attribute_from_enabled;
+  int sip_attribute_from_imsi_enabled;
+  int sip_attribute_to_enabled;
+  int sip_attribute_to_imsi_enabled;
+
   int stun_opportunistic_tls_enabled;
   int stun_max_packets_extra_dissection;
+  int rtp_max_packets_extra_dissection;
   int stun_mapped_address_enabled;
   int stun_response_origin_enabled;
   int stun_other_address_enabled;
   int stun_relayed_address_enabled;
   int stun_peer_address_enabled;
 
+  int bittorrent_hash_enabled;
+
+  int ssdp_metadata_enabled;
+
   int dns_subclassification_enabled;
   int dns_parse_response_enabled;
 
   int http_parse_response_enabled;
+  int http_subclassification_enabled;
 
   int ookla_aggressiveness;
 
@@ -264,12 +324,23 @@ struct ndpi_detection_module_config_struct {
 
   int rtp_search_for_stun;
 
+  int openvpn_heuristics;
+  int openvpn_heuristics_num_msgs;
+  int openvpn_subclassification_by_ip;
+
+  int wireguard_subclassification_by_ip;
+
   NDPI_PROTOCOL_BITMASK debug_bitmask;
   NDPI_PROTOCOL_BITMASK ip_list_bitmask;
+  NDPI_PROTOCOL_BITMASK monitoring;
+
+  NDPI_PROTOCOL_BITMASK flowrisk_bitmask;
+  NDPI_PROTOCOL_BITMASK flowrisk_info_bitmask;
 
   int flow_risk_lists_enabled;
   int risk_anonymous_subscriber_list_icloudprivaterelay_enabled;
   int risk_anonymous_subscriber_list_protonvpn_enabled;
+  int risk_anonymous_subscriber_list_tor_exit_nodes_enabled;
   int risk_crawler_bot_list_enabled;
 };
 
@@ -323,7 +394,7 @@ struct ndpi_detection_module_struct {
    * update automa_type above
    */
 
-  ndpi_str_hash *malicious_ja3_hashmap, *malicious_sha1_hashmap;
+  ndpi_str_hash *malicious_ja4_hashmap, *malicious_sha1_hashmap, *tcp_fingerprint_hashmap;
 
   ndpi_list *trusted_issuer_dn;
 
@@ -333,11 +404,7 @@ struct ndpi_detection_module_struct {
   /* *** If you add a new Patricia tree, please update ptree_type above! *** */
 
   struct {
-#ifdef USE_LEGACY_AHO_CORASICK
-    ndpi_automa hostnames, hostnames_shadow;
-#else
     ndpi_domain_classify *sc_hostnames, *sc_hostnames_shadow;
-#endif
     void *ipAddresses, *ipAddresses_shadow; /* Patricia */
     void *ipAddresses6, *ipAddresses6_shadow; /* Patricia IPv6*/
     u_int8_t categories_loaded;
@@ -354,6 +421,9 @@ struct ndpi_detection_module_struct {
   /* NDPI_PROTOCOL_OOKLA */
   struct ndpi_lru_cache *ookla_cache;
 
+  /* NDPI_PROTOCOL_SIGNAL */
+  struct ndpi_lru_cache *signal_cache;
+
   /* NDPI_PROTOCOL_BITTORRENT */
   struct ndpi_lru_cache *bittorrent_cache;
 
@@ -387,7 +457,7 @@ struct ndpi_detection_module_struct {
 
   /* Current packet */
   struct ndpi_packet_struct packet;
-  const struct ndpi_flow_input_info *input_info;
+  struct ndpi_flow_input_info *input_info;
 
 #ifdef HAVE_NBPF
   u_int8_t num_nbpf_custom_proto;
@@ -397,6 +467,7 @@ struct ndpi_detection_module_struct {
   u_int16_t max_payload_track_len;
 
   ndpi_str_hash *public_domain_suffixes;
+  struct ndpi_address_cache *address_cache;
 };
 
 
@@ -543,10 +614,6 @@ struct ndpi_detection_module_struct {
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP_OR_UDP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION	(NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP_OR_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_NO_TCP_RETRANSMISSION | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION	(NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_NO_TCP_RETRANSMISSION | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
 
-
-
-
-
 /* Generic */
 
 char *strptime(const char *s, const char *format, struct tm *tm);
@@ -588,7 +655,7 @@ u_int8_t is_a_common_alpn(struct ndpi_detection_module_struct *ndpi_str,
 
 int64_t asn1_ber_decode_length(const unsigned char *payload, int payload_len, u_int16_t *value_len);
 
-u_int8_t ips_match(u_int32_t src, u_int32_t dst,
+u_int8_t ndpi_ips_match(u_int32_t src, u_int32_t dst,
 		   u_int32_t net, u_int32_t num_bits);
 
 u_int8_t ends_with(struct ndpi_detection_module_struct *ndpi_struct,
@@ -599,6 +666,9 @@ u_int ndpi_search_tcp_or_udp_raw(struct ndpi_detection_module_struct *ndpi_struc
 				 u_int32_t saddr, u_int32_t daddr);
 
 char* ndpi_intoav4(unsigned int addr, char* buf, u_int16_t bufLen);
+char* ndpi_intoav6(struct ndpi_in6_addr *addr, char* buf, u_int16_t bufLen);
+  
+int is_flow_addr_informative(const struct ndpi_flow_struct *flow);
 
 u_int16_t icmp4_checksum(u_int8_t const * const buf, size_t len);
 
@@ -608,16 +678,22 @@ ndpi_risk_enum ndpi_network_risk_ptree_match(struct ndpi_detection_module_struct
 int load_protocols_file_fd(struct ndpi_detection_module_struct *ndpi_mod, FILE *fd);
 int load_categories_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd, void *user_data);
 int load_malicious_sha1_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd);
-int load_malicious_ja3_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd);
+int load_malicious_ja4_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd);
 int load_risk_domain_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd);
 int load_config_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd);
 int load_category_file_fd(struct ndpi_detection_module_struct *ndpi_str,
 			  FILE *fd, ndpi_protocol_category_t category_id);
 
-u_int64_t fpc_dns_cache_key_from_dns_info(struct ndpi_flow_struct *flow);
+u_int64_t fpc_dns_cache_key_from_flow(struct ndpi_flow_struct *flow);
 
+bool ndpi_cache_address(struct ndpi_detection_module_struct *ndpi_struct,
+			ndpi_ip_addr_t ip_addr, char *hostname,
+			u_int32_t epoch_now, u_int32_t ttl);
 
-/* TLS */
+int is_monitoring_enabled(struct ndpi_detection_module_struct *ndpi_str, int protoId);
+int is_flowrisk_info_enabled(struct ndpi_detection_module_struct *ndpi_str, ndpi_risk_enum flowrisk_id);
+
+  /* TLS */
 int processClientServerHello(struct ndpi_detection_module_struct *ndpi_struct,
                              struct ndpi_flow_struct *flow, uint32_t quic_version);
 void processCertificateElements(struct ndpi_detection_module_struct *ndpi_struct,
@@ -628,11 +704,8 @@ void switch_to_tls(struct ndpi_detection_module_struct *ndpi_struct,
 int is_dtls(const u_int8_t *buf, u_int32_t buf_len, u_int32_t *block_len);
 void switch_extra_dissection_to_tls(struct ndpi_detection_module_struct *ndpi_struct,
 				    struct ndpi_flow_struct *flow);
-
-/* HTTP */
-void http_process_user_agent(struct ndpi_detection_module_struct *ndpi_struct,
-                             struct ndpi_flow_struct *flow,
-                             const u_int8_t *ua_ptr, u_int16_t ua_ptr_len);
+void switch_extra_dissection_to_tls_obfuscated_heur(struct ndpi_detection_module_struct* ndpi_struct,
+                                                    struct ndpi_flow_struct* flow);
 
 /* OOKLA */
 int ookla_search_into_cache(struct ndpi_detection_module_struct* ndpi_struct,
@@ -640,11 +713,18 @@ int ookla_search_into_cache(struct ndpi_detection_module_struct* ndpi_struct,
 void ookla_add_to_cache(struct ndpi_detection_module_struct *ndpi_struct,
                         struct ndpi_flow_struct *flow);
 
+/* SIGNAL */
+int signal_search_into_cache(struct ndpi_detection_module_struct* ndpi_struct,
+                            struct ndpi_flow_struct* flow);
+void signal_add_to_cache(struct ndpi_detection_module_struct *ndpi_struct,
+                        struct ndpi_flow_struct *flow);
+
 /* QUIC */
 int quic_len(const uint8_t *buf, uint64_t *value);
 int quic_len_buffer_still_required(uint8_t value);
 int is_version_with_var_int_transport_params(uint32_t version);
 int is_version_with_tls(uint32_t version);
+int is_quic_ver_greater_than(uint32_t version, uint8_t min_version);
 void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
                   struct ndpi_flow_struct *flow,
                   const u_int8_t *crypto_data, uint32_t crypto_data_len);
@@ -660,7 +740,7 @@ const uint8_t *get_crypto_data(struct ndpi_detection_module_struct *ndpi_struct,
 int is_valid_rtp_payload_type(uint8_t type);
 int is_rtp_or_rtcp(struct ndpi_detection_module_struct *ndpi_struct,
                    const u_int8_t *payload, u_int16_t payload_len, u_int16_t *seq);
-u_int8_t rtp_get_stream_type(u_int8_t payloadType, ndpi_multimedia_flow_type *s_type);
+u_int8_t rtp_get_stream_type(u_int8_t payloadType, u_int8_t *s_type, u_int16_t sub_proto);
 
 /* Bittorrent */
 u_int64_t make_bittorrent_host_key(struct ndpi_flow_struct *flow, int client, int offset);
@@ -673,7 +753,7 @@ int search_into_bittorrent_cache(struct ndpi_detection_module_struct *ndpi_struc
 int is_stun(struct ndpi_detection_module_struct *ndpi_struct,
             struct ndpi_flow_struct *flow,
             u_int16_t *app_proto);
-void switch_extra_dissection_to_stun(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow);
+void switch_extra_dissection_to_stun(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow, int std_callback);
 
 /* TPKT */
 int tpkt_verify_hdr(const struct ndpi_packet_struct * const packet);
@@ -725,10 +805,11 @@ void init_lotus_notes_dissector(struct ndpi_detection_module_struct *ndpi_struct
 void init_mail_imap_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_mail_pop_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_mail_smtp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
-void init_maplestory_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_nexon_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_megaco_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_mgcp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_mining_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_mikrotik_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_mms_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_monero_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_nats_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
@@ -765,10 +846,11 @@ void init_smb_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int3
 void init_snmp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_socrates_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_socks_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_sonos_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_spotify_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_ssh_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_tls_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
-void init_starcraft_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_blizzard_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_steam_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_stun_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_syslog_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
@@ -788,7 +870,6 @@ void init_vnc_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int3
 void init_vxlan_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_warcraft3_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_whois_das_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
-void init_world_of_warcraft_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_world_of_kung_fu_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_xbox_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_xdmcp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
@@ -932,6 +1013,20 @@ void init_ripe_atlas_dissector(struct ndpi_detection_module_struct *ndpi_struct,
 void init_cloudflare_warp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_nano_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
 void init_openwire_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_cnp_ip_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_atg_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_trdp_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_lustre_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_dingtalk_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_paltalk_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_dicom_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_lagofast_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+void init_gearup_booster_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id);
+
+
+#ifdef CUSTOM_NDPI_PROTOCOLS
+  #include "../../../nDPI-custom/custom_ndpi_private.h"
+#endif
 
 #endif
 
