@@ -25,6 +25,7 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_CISCOVPN
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 /* ****************************************************************** */
 
@@ -38,35 +39,29 @@ static void ndpi_int_ciscovpn_add_connection(struct ndpi_detection_module_struct
 static void ndpi_search_ciscovpn(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
 {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
-  u_int16_t udport = 0, usport = 0;
+  u_int16_t udport, usport;
 
   NDPI_LOG_DBG(ndpi_struct, "search CISCOVPN\n");
 
-  if(packet->udp != NULL) {
-    usport = ntohs(packet->udp->source), udport = ntohs(packet->udp->dest);
-    NDPI_LOG_DBG2(ndpi_struct, "calculated CISCOVPN over udp ports\n");
-  }
+  usport = ntohs(packet->udp->source), udport = ntohs(packet->udp->dest);
 
-  if(
-	  (
-	   (usport == 10000 && udport == 10000)
-	   &&
-	   (packet->payload_packet_len >= 4) &&
-	   (packet->payload[0] == 0xfe &&
-	    packet->payload[1] == 0x57 &&
-	    packet->payload[2] == 0x7e &&
-	    packet->payload[3] == 0x2b)
-	   )
-	  )
-    {
+  if((usport == 10000 && udport == 10000)) {
+    if((packet->payload_packet_len >= 4) &&
+       (packet->payload[0] == 0xfe &&
+	packet->payload[1] == 0x57 &&
+	packet->payload[2] == 0x7e &&
+	packet->payload[3] == 0x2b)
+       ) {
       /* This is a good query  fe577e2b */
       NDPI_LOG_INFO(ndpi_struct, "found CISCOVPN\n");
       ndpi_int_ciscovpn_add_connection(ndpi_struct, flow);
       return;
-    } 
-
-  if(flow->num_processed_pkts > 5)
-    NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+    }
+    
+    if(flow->num_processed_pkts > 5)
+      NDPI_EXCLUDE_PROTO(ndpi_struct, flow);    
+  } else
+    NDPI_EXCLUDE_PROTO(ndpi_struct, flow);  
 }
 
 
@@ -75,7 +70,7 @@ void init_ciscovpn_dissector(struct ndpi_detection_module_struct *ndpi_struct, u
   ndpi_set_bitmask_protocol_detection("CiscoVPN", ndpi_struct, *id,
 				      NDPI_PROTOCOL_CISCOVPN,
 				      ndpi_search_ciscovpn,
-				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
+				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP_WITH_PAYLOAD,
 				      SAVE_DETECTION_BITMASK_AS_UNKNOWN,
 				      ADD_TO_DETECTION_BITMASK);
   *id += 1;
