@@ -23,26 +23,39 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_CORBA
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 static void ndpi_int_corba_add_connection(struct ndpi_detection_module_struct
-					  *ndpi_struct, struct ndpi_flow_struct *flow)
+                                          *ndpi_struct, struct ndpi_flow_struct *flow)
 {
+  NDPI_LOG_INFO(ndpi_struct, "found Corba\n");
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_CORBA, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 static void ndpi_search_corba(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
 {
-  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
+  struct ndpi_packet_struct const * const packet = &ndpi_struct->packet;
 
-  NDPI_LOG_DBG(ndpi_struct, "search for CORBA\n");
-  if(packet->tcp != NULL) {
-    NDPI_LOG_DBG2(ndpi_struct, "calculating CORBA over tcp\n");
-    /* Corba General Inter-ORB Protocol -> GIOP */
-    if(packet->payload_packet_len >= 24 &&
-       memcmp(packet->payload, "GIOP", 4) == 0) {
-      NDPI_LOG_INFO(ndpi_struct, "found corba\n");
+  NDPI_LOG_DBG(ndpi_struct, "search for Corba\n");
+
+  if (packet->tcp != NULL && packet->payload_packet_len >= 24) {
+    /* General Inter-ORB Protocol -> GIOP
+     * Zipped Inter-ORB Protocol  -> ZIOP */
+    if ((memcmp(packet->payload, "GIOP", 4) == 0) ||
+        (memcmp(packet->payload, "ZIOP", 4) == 0))
+    {
       ndpi_int_corba_add_connection(ndpi_struct, flow);
+      return;
+    }
+  } 
+  else if (packet->udp != NULL && packet->payload_packet_len > 32) {
+    /* Unreliable Multicast Inter-ORB Protocol -> MIOP */
+    if (memcmp(packet->payload, "MIOP", 4) == 0) {
+      ndpi_int_corba_add_connection(ndpi_struct, flow);
+      return;
     }
   }
+
+  NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
 }
 
 
@@ -51,7 +64,7 @@ void init_corba_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_in
   ndpi_set_bitmask_protocol_detection("Corba", ndpi_struct, *id,
 				      NDPI_PROTOCOL_CORBA,
 				      ndpi_search_corba,
-				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
+				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
 				      SAVE_DETECTION_BITMASK_AS_UNKNOWN,
 				      ADD_TO_DETECTION_BITMASK);
   *id += 1;
