@@ -1,7 +1,7 @@
 /*
  * mail_smtp.c
  *
- * Copyright (C) 2011-22 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  * Copyright (C) 2009-11 - ipoque GmbH
  *
  * This file is part of nDPI, an open source deep packet inspection
@@ -28,6 +28,7 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_MAIL_SMTP
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 
 #define SMTP_BIT_220		0x01
@@ -48,8 +49,6 @@
 
 /* #define SMTP_DEBUG 1 */
 
-extern int processTLSBlock(struct ndpi_detection_module_struct *ndpi_struct,
-                           struct ndpi_flow_struct *flow);
 
 static void ndpi_int_mail_smtp_add_connection(struct ndpi_detection_module_struct
 					      *ndpi_struct, struct ndpi_flow_struct *flow) {
@@ -68,7 +67,7 @@ static void smtpInitExtraPacketProcessing(struct ndpi_flow_struct *flow);
 /* **************************************** */
 
 static void get_credentials_auth_plain(struct ndpi_detection_module_struct *ndpi_struct,
-				       struct ndpi_flow_struct *flow,
+                                       struct ndpi_flow_struct *flow,
 				       const u_int8_t *line, u_int16_t line_len)
 {
   u_int8_t buf[255];
@@ -118,8 +117,8 @@ static void get_credentials_auth_plain(struct ndpi_detection_module_struct *ndpi
 
 /* **************************************** */
 
-void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
-			       struct ndpi_flow_struct *flow) {
+static void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
+				      struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
   NDPI_LOG_DBG(ndpi_struct, "search mail_smtp\n");
@@ -152,7 +151,8 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 		   && (packet->line[a].ptr[i+1] != '\n')) {
 		  len = i-4;
 		  /* Copy result for nDPI apps */
-		  ndpi_hostname_sni_set(flow, &packet->line[a].ptr[4], len);
+		  ndpi_hostname_sni_set(flow, &packet->line[a].ptr[4], len, NDPI_HOSTNAME_NORM_ALL);
+		  NDPI_LOG_DBG(ndpi_struct, "SMTP: hostname [%s]\n", flow->host_server_name);
 
 		  if (ndpi_match_hostname_protocol(ndpi_struct, flow, NDPI_PROTOCOL_MAIL_SMTP,
 						   flow->host_server_name,
@@ -315,6 +315,27 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
     }
   }
 
+      if(packet->line[a].len >= 14) {
+        if((packet->line[a].ptr[0] == 'X' || packet->line[a].ptr[0] == 'x')
+            && (packet->line[a].ptr[1] == '-' || packet->line[a].ptr[1] == '-')
+            && (packet->line[a].ptr[2] == 'A' || packet->line[a].ptr[2] == 'a')
+            && (packet->line[a].ptr[3] == 'N' || packet->line[a].ptr[3] == 'n')
+            && (packet->line[a].ptr[4] == 'O' || packet->line[a].ptr[4] == 'o')
+            && (packet->line[a].ptr[5] == 'N' || packet->line[a].ptr[5] == 'n')
+            && (packet->line[a].ptr[6] == 'Y' || packet->line[a].ptr[6] == 'y')
+            && (packet->line[a].ptr[7] == 'M' || packet->line[a].ptr[6] == 'm')
+            && (packet->line[a].ptr[8] == 'O' || packet->line[a].ptr[6] == 'o')
+            && (packet->line[a].ptr[9] == 'U' || packet->line[a].ptr[6] == 'u')
+            && (packet->line[a].ptr[10] == 'S' || packet->line[a].ptr[6] == 's')
+            && (packet->line[a].ptr[11] == 'T' || packet->line[a].ptr[6] == 't')
+            && (packet->line[a].ptr[12] == 'L' || packet->line[a].ptr[6] == 'l')
+            && (packet->line[a].ptr[13] == 'S' || packet->line[a].ptr[7] == 's')) {
+          flow->l4.tcp.smtp_command_bitmask |= SMTP_BIT_STARTTLS;
+          flow->l4.tcp.ftp_imap_pop_smtp.auth_tls = 1;
+          flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 0;
+        }
+      }
+
       if(packet->line[a].len >= 4) {
 	if((packet->line[a].ptr[0] == 'D' || packet->line[a].ptr[0] == 'd')
 	   && (packet->line[a].ptr[1] == 'A' || packet->line[a].ptr[1] == 'a')
@@ -375,7 +396,7 @@ void ndpi_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_struct,
     return;
   }
 
-  if((!flow->check_extra_packets) || (flow->packet_counter > 12))
+  if((!flow->extra_packets_func) || (flow->packet_counter > 12))
     NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
 }
 
@@ -385,41 +406,45 @@ int ndpi_extra_search_mail_smtp_tcp(struct ndpi_detection_module_struct *ndpi_st
 				    struct ndpi_flow_struct *flow)
 {
   struct ndpi_packet_struct * const packet = &ndpi_struct->packet;
-  int rc = 0;
+  int rc;
 
-  if (flow->l4.tcp.smtp_command_bitmask & SMTP_BIT_STARTTLS &&
-      packet->payload_packet_len > 5)
-  {
-    uint8_t const * const block = &packet->payload[5];
-    uint8_t const * const p = &packet->payload[0];
-    uint16_t const block_len = packet->payload_packet_len - 5;
-    uint16_t const l = packet->payload_packet_len;
+  if(flow->l4.tcp.smtp_command_bitmask & SMTP_BIT_STARTTLS) {
 
-    packet->payload = block;
-    packet->payload_packet_len = block_len;
+    /* RFC 3207:
+       "After the client gives the STARTTLS command, the server responds with
+        one of the following reply codes:
+         220 Ready to start TLS
+         501 Syntax error (no parameters allowed)
+         454 TLS not available due to temporary reason"
+    */
 
-    if (processTLSBlock(ndpi_struct, flow) != 0) {
+    if(ndpi_struct->cfg.smtp_opportunistic_tls_enabled &&
+       packet->payload_packet_len > 3 && memcmp(packet->payload, "220", 3) == 0) {
       rc = 1;
-    }
-
-    packet->payload = p;
-    packet->payload_packet_len = l;
-
-    /* STARTTLS may be followed by a 220 - Service ready */
-    if (rc == 0 && memcmp(packet->payload, "220", 3) != 0)
-    {
-      flow->l4.tcp.ftp_imap_pop_smtp.auth_done = 1;
-      if (flow->guessed_host_protocol_id == NDPI_PROTOCOL_UNKNOWN) {
-        ndpi_set_detected_protocol(ndpi_struct, flow,
-                                   NDPI_PROTOCOL_MAIL_SMTPS, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+      /* Switch classification to SMTPS, keeping the hostname sub-classification (if any) */
+      if(flow->detected_protocol_stack[0] != NDPI_PROTOCOL_UNKNOWN &&
+	 flow->detected_protocol_stack[0] != NDPI_PROTOCOL_MAIL_SMTP) {
+	ndpi_set_detected_protocol(ndpi_struct, flow,
+                                   flow->detected_protocol_stack[0], NDPI_PROTOCOL_MAIL_SMTPS, NDPI_CONFIDENCE_DPI);
+	/* Now it is safe to write to `flow->protos.tls_quic` union */
+	flow->protos.tls_quic.subprotocol_detected = 1;
       } else {
         ndpi_set_detected_protocol(ndpi_struct, flow,
-                                   flow->guessed_host_protocol_id, NDPI_PROTOCOL_MAIL_SMTPS, NDPI_CONFIDENCE_DPI);
+                                   NDPI_PROTOCOL_MAIL_SMTPS, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
       }
+      NDPI_LOG_DBG(ndpi_struct, "Switching to [%d/%d]\n",
+                   flow->detected_protocol_stack[0], flow->detected_protocol_stack[1]);
+      /* We are done (in SMTP dissector): delegating TLS... */
+      switch_extra_dissection_to_tls(ndpi_struct, flow);
+    } else {
+      rc = 0; /* Something went wrong. Stop extra dissection */
     }
+
   } else {
     ndpi_search_mail_smtp_tcp(ndpi_struct, flow);
-    rc = (flow->l4.tcp.ftp_imap_pop_smtp.password[0] == '\0') ? 1 : 0;
+    rc = ((flow->l4.tcp.ftp_imap_pop_smtp.password[0] == '\0') &&
+          (flow->l4.tcp.ftp_imap_pop_smtp.auth_tls == 1 ||
+           flow->l4.tcp.ftp_imap_pop_smtp.auth_done == 0)) ? 1 : 0;
   }
 
 #ifdef SMTP_DEBUG
@@ -438,7 +463,6 @@ static void smtpInitExtraPacketProcessing(struct ndpi_flow_struct *flow) {
   printf("**** %s(%u)\n", __FUNCTION__, ++num);
 #endif
 
-  flow->check_extra_packets = 1;
   /* At most 12 packets should almost always be enough */
   flow->max_extra_packets_to_check = 12;
   flow->extra_packets_func = ndpi_extra_search_mail_smtp_tcp;
@@ -447,8 +471,8 @@ static void smtpInitExtraPacketProcessing(struct ndpi_flow_struct *flow) {
 /* **************************************** */
 
 void init_mail_smtp_dissector(struct ndpi_detection_module_struct *ndpi_struct,
-			      u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask) {
-  ndpi_set_bitmask_protocol_detection("MAIL_SMTP", ndpi_struct, detection_bitmask, *id,
+			      u_int32_t *id) {
+  ndpi_set_bitmask_protocol_detection("MAIL_SMTP", ndpi_struct, *id,
 				      NDPI_PROTOCOL_MAIL_SMTP,
 				      ndpi_search_mail_smtp_tcp,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
