@@ -2,7 +2,7 @@
  * non_tcp_udp.c
  *
  * Copyright (C) 2009-11 - ipoque GmbH
- * Copyright (C) 2011-22 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -41,6 +41,8 @@
 static void ndpi_search_in_non_tcp_udp(struct ndpi_detection_module_struct
 				       *ndpi_struct, struct ndpi_flow_struct *flow)
 {
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
+
   switch (flow->l4_proto) {
   case NDPI_IPSEC_PROTOCOL_ESP:
   case NDPI_IPSEC_PROTOCOL_AH:
@@ -53,6 +55,43 @@ static void ndpi_search_in_non_tcp_udp(struct ndpi_detection_module_struct
 
   case NDPI_ICMP_PROTOCOL_TYPE:
     set_protocol_and_bmask(NDPI_PROTOCOL_IP_ICMP);
+
+    if(packet->payload_packet_len < sizeof(struct ndpi_icmphdr)) {
+      char buf[64];
+
+      snprintf(buf, sizeof(buf), "Packet too short (%d vs %u)",
+               packet->payload_packet_len, (unsigned int)sizeof(struct ndpi_icmphdr));
+      ndpi_set_risk(ndpi_struct, flow, NDPI_MALFORMED_PACKET, buf);
+    } else {
+      u_int8_t icmp_type = (u_int8_t)packet->payload[0];
+      u_int8_t icmp_code = (u_int8_t)packet->payload[1];
+
+      /* https://www.iana.org/assignments/icmp-parameters/icmp-parameters.xhtml */
+      if(((icmp_type >= 44) && (icmp_type <= 252))
+         || (icmp_code > 15)) {
+        char buf[64];
+
+        snprintf(buf, sizeof(buf), "Invalid type (%u)/code(%u)",
+                 icmp_type, icmp_code);
+
+        ndpi_set_risk(ndpi_struct, flow, NDPI_MALFORMED_PACKET, buf);
+      }
+
+      if(packet->payload_packet_len > sizeof(struct ndpi_icmphdr)) {
+        if(ndpi_struct->cfg.compute_entropy && (flow->skip_entropy_check == 0)) {
+          flow->entropy = ndpi_entropy(packet->payload + sizeof(struct ndpi_icmphdr),
+                                       packet->payload_packet_len - sizeof(struct ndpi_icmphdr));
+          ndpi_entropy2risk(ndpi_struct, flow);
+        }
+
+        u_int16_t chksm = icmp4_checksum(packet->payload, packet->payload_packet_len);
+
+        if(chksm) {
+          ndpi_set_risk(ndpi_struct, flow, NDPI_MALFORMED_PACKET, "Invalid ICMP checksum");
+        }
+      }
+    }
+
     break;
 
   case NDPI_IGMP_PROTOCOL_TYPE:
@@ -81,6 +120,30 @@ static void ndpi_search_in_non_tcp_udp(struct ndpi_detection_module_struct
 
   case NDPI_ICMPV6_PROTOCOL_TYPE:
     set_protocol_and_bmask(NDPI_PROTOCOL_IP_ICMPV6);
+
+    if(packet->payload_packet_len < sizeof(struct ndpi_icmp6hdr)) {
+      char buf[64];
+
+      snprintf(buf, sizeof(buf), "Packet too short (%d vs %u)",
+               packet->payload_packet_len, (unsigned int)sizeof(struct ndpi_icmp6hdr));
+
+      ndpi_set_risk(ndpi_struct, flow, NDPI_MALFORMED_PACKET, buf);
+    } else {
+      u_int8_t icmp6_type = (u_int8_t)packet->payload[0];
+      u_int8_t icmp6_code = (u_int8_t)packet->payload[1];
+
+      /* https://en.wikipedia.org/wiki/Internet_Control_Message_Protocol_for_IPv6 */
+      if(((icmp6_type >= 5) && (icmp6_type <= 127))
+         || ((icmp6_code >= 156) && (icmp6_type != 255))) {
+        char buf[64];
+
+        snprintf(buf, sizeof(buf), "Invalid type (%u)/code(%u)",
+                 icmp6_type, icmp6_code);
+
+        ndpi_set_risk(ndpi_struct, flow, NDPI_MALFORMED_PACKET, buf);
+      }
+    }
+
     break;
 
   case NDPI_PIM_PROTOCOL_TYPE:
