@@ -28,10 +28,12 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_TELEGRAM
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 static void ndpi_int_telegram_add_connection(struct ndpi_detection_module_struct
-                                             *ndpi_struct, struct ndpi_flow_struct *flow) {
-  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_TELEGRAM, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
+                                             *ndpi_struct, struct ndpi_flow_struct *flow,
+					     ndpi_confidence_t confidence) {
+  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_TELEGRAM, NDPI_PROTOCOL_UNKNOWN, confidence);
   NDPI_LOG_INFO(ndpi_struct, "found telegram\n");
 }
 
@@ -50,18 +52,15 @@ static void ndpi_search_telegram(struct ndpi_detection_module_struct *ndpi_struc
   NDPI_LOG_DBG(ndpi_struct, "search telegram\n");
 
   if(packet->tcp != NULL) {
-    if(packet->payload_packet_len > 56) {
-      u_int16_t dport = ntohs(packet->tcp->dest);
-      /* u_int16_t sport = ntohs(packet->tcp->source); */
-
-      if(packet->payload[0] == 0xef && (dport == 443 || dport == 80 || dport == 25)) {
-        if(packet->payload[1] == 0x7f) {
-          ndpi_int_telegram_add_connection(ndpi_struct, flow);
-        } else if(packet->payload[1]*4 <= packet->payload_packet_len - 1) {
-          ndpi_int_telegram_add_connection(ndpi_struct, flow);
-        }
-        return;
-      }
+    /* With MTProto 2.0 telegram via app is no longer TLS-based (althought based on TCP/443) so
+       we need to detect it with Telegram IPs.
+       Basically, we want a fast classification by ip. Note that, real Telegram traffic over
+       TLS (i.e. Telegram Web) is correctly classified as TLS/Telegram because TLS dissector
+       already kicked in.
+       Let's check every port for the time being */
+    if(flow->guessed_protocol_id_by_ip == NDPI_PROTOCOL_TELEGRAM) {
+      ndpi_int_telegram_add_connection(ndpi_struct, flow, NDPI_CONFIDENCE_MATCH_BY_IP);
+      return;
     }
   } else if(packet->udp != NULL) {
     /*
@@ -74,7 +73,7 @@ static void ndpi_search_telegram(struct ndpi_detection_module_struct *ndpi_struc
       u_int16_t sport = ntohs(packet->udp->source), dport = ntohs(packet->udp->dest);
 
       if(is_telegram_port_range(sport) || is_telegram_port_range(dport)) {
-	u_int i=0, found = 0;
+	u_int i, found = 0;
 
 	for(i=0; i<packet->payload_packet_len; i++) {
 	  if(packet->payload[i] == 0xFF) {
@@ -93,7 +92,15 @@ static void ndpi_search_telegram(struct ndpi_detection_module_struct *ndpi_struc
 	}
 
 	if(found == 12)	{
-	  ndpi_int_telegram_add_connection(ndpi_struct, flow);
+	  ndpi_int_telegram_add_connection(ndpi_struct, flow, NDPI_CONFIDENCE_DPI);
+	  /* It seems this kind of traffic is used:
+             * for "normal" stuff (at least years ago... and now? TODO)
+             * for calls, as a custom encapsulation of STUN/DTLS/RTP packets
+             Since we are not able to tell the former from the latter, always
+             switch to STUN dissection. If we find STUN/DTLS/RTP stuff we will
+             update the classification to something like STUN/Telegram_voip,
+             otherwise it will remain Telegram */
+	  switch_extra_dissection_to_stun(ndpi_struct, flow, 0);
 	  return;
 	}
       }
