@@ -106,6 +106,38 @@ static void ndpi_search_websocket(struct ndpi_detection_module_struct *ndpi_stru
   NDPI_LOG_DBG(ndpi_struct, "search WEBSOCKET\n");
   ndpi_check_websocket(ndpi_struct, flow);
 
+  // Check also some HTTP headers indicating an upcoming WebSocket connection
+  if (flow->detected_protocol_stack[0] == NDPI_PROTOCOL_HTTP &&
+      flow->detected_protocol_stack[1] != NDPI_PROTOCOL_WEBSOCKET)
+  {
+    struct ndpi_packet_struct const * const packet = &ndpi_struct->packet;
+    uint16_t i;
+
+    NDPI_PARSE_PACKET_LINE_INFO(ndpi_struct, flow, packet);
+    for (i = 0; i < packet->parsed_lines; i++) {
+      if (LINE_STARTS(packet->line[i], "upgrade:") != 0 &&
+          LINE_ENDS(packet->line[i], "websocket") != 0)
+      {
+        ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_WEBSOCKET,
+                                   NDPI_PROTOCOL_HTTP, NDPI_CONFIDENCE_DPI);
+      } else if (LINE_STARTS(packet->line[i], "sec-websocket") != 0) {
+        ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_WEBSOCKET,
+                                   NDPI_PROTOCOL_HTTP, NDPI_CONFIDENCE_DPI);
+        if (ndpi_strncasestr((const char *)packet->line[i].ptr, "chisel",
+                             packet->line[i].len) != NULL)
+        {
+          ndpi_set_risk(ndpi_struct, flow, NDPI_OBFUSCATED_TRAFFIC,
+                        "Obfuscated SSH-in-HTTP-WebSocket traffic");
+        }
+      }
+    }
+    if (i == packet->parsed_lines)
+    {
+      NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+      return;
+    }
+  }
+
   return;
 }
 
