@@ -23,12 +23,12 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_ZABBIX
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 /* *************************************************** */
 
 static void ndpi_int_zabbix_add_connection(struct ndpi_detection_module_struct *ndpi_struct,
-					   struct ndpi_flow_struct *flow/* , */
-					   /* ndpi_protocol_type_t protocol_type */) {
+					   struct ndpi_flow_struct *flow) {
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_ZABBIX, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
