@@ -95,16 +95,23 @@ int serializerUnitTest() {
 
     for(i=0; i<16; i++) {
       char kbuf[32], vbuf[32];
-      ndpi_snprintf(kbuf, sizeof(kbuf), "Key %d", i);
+      int j = 0;
       ndpi_snprintf(vbuf, sizeof(vbuf), "Value %d", i);
-      assert(ndpi_serialize_uint32_uint32(&serializer, i, i*i) != -1);
-      assert(ndpi_serialize_uint32_string(&serializer, i, "Data") != -1);
+      assert(ndpi_serialize_uint32_uint32(&serializer, j++, i*i) != -1);
+      assert(ndpi_serialize_uint32_string(&serializer, j++, "Data") != -1);
+      ndpi_snprintf(kbuf, sizeof(kbuf), "Key %d", j++);
       assert(ndpi_serialize_string_string(&serializer, kbuf, vbuf) != -1);
+      ndpi_snprintf(kbuf, sizeof(kbuf), "Key %d", j++);
       assert(ndpi_serialize_string_uint32(&serializer, kbuf, i*i) != -1);
+      ndpi_snprintf(kbuf, sizeof(kbuf), "Key %d", j++);
       assert(ndpi_serialize_string_float(&serializer,  kbuf, (float)(i*i), "%f") != -1);
-      if (fmt != ndpi_serialization_format_tlv)
+      if (fmt != ndpi_serialization_format_tlv) {
+        ndpi_snprintf(kbuf, sizeof(kbuf), "Key %d", j++);
         assert(ndpi_serialize_string_double(&serializer, kbuf, ((double)(FLT_MAX))*2, "%lf") != -1);
+      }
+      ndpi_snprintf(kbuf, sizeof(kbuf), "Key %d", j++);
       assert(ndpi_serialize_string_int64(&serializer,  kbuf, INT64_MAX) != -1);
+      assert(ndpi_serialize_string_string(&serializer, "utf-8", "küche") != -1);
       if ((i&0x3) == 0x3) ndpi_serialize_end_of_record(&serializer);
     }
 
@@ -282,11 +289,14 @@ int serializeProtoUnitTest(void)
     }
     assert(ndpi_init_serializer(&serializer, fmt) != -1);
 
-    ndpi_protocol ndpi_proto = { .master_protocol = NDPI_PROTOCOL_TLS,
-                                 .app_protocol = NDPI_PROTOCOL_FACEBOOK,
-                                 .protocol_by_ip = NDPI_PROTOCOL_FACEBOOK,
-                                 .category = NDPI_PROTOCOL_CATEGORY_SOCIAL_NETWORK };
+    ndpi_protocol ndpi_proto;
     ndpi_risk risks = 0;
+
+    ndpi_proto.proto.master_protocol = NDPI_PROTOCOL_TLS,
+      ndpi_proto.proto.app_protocol = NDPI_PROTOCOL_FACEBOOK,
+      ndpi_proto.protocol_by_ip = NDPI_PROTOCOL_FACEBOOK,
+      ndpi_proto.category = NDPI_PROTOCOL_CATEGORY_SOCIAL_NETWORK;
+       
     NDPI_SET_BIT(risks, NDPI_MALFORMED_PACKET);
     NDPI_SET_BIT(risks, NDPI_TLS_WEAK_CIPHER);
     NDPI_SET_BIT(risks, NDPI_TLS_OBSOLETE_VERSION);
