@@ -1,5 +1,4 @@
 #!/usr/bin/env bash
-
 set -e
 
 cd "$(dirname "${0}")" || exit 1
@@ -8,6 +7,7 @@ cd "$(dirname "${0}")" || exit 1
 DEST=../src/lib/inc_generated/ndpi_ethereum_match.c.inc
 TMP=/tmp/ethereum
 LIST=/tmp/ethereum.list
+LIST_MERGED=/tmp/ethereum.list_m
 ORIGIN="https://raw.githubusercontent.com/ethereum/go-ethereum/master/params/bootnodes.go"
 
 
@@ -17,12 +17,14 @@ check_http_response "${http_response}"
 is_file_empty "${TMP}"
 
 echo "(2) Processing IP addresses..."
-grep 'enode' $TMP | grep -v '^/' | grep ':' | cut -d '@' -f 2 | cut -d ':' -f 1 > $LIST
+grep 'enode' $TMP | grep -v '^/' | grep ':' | cut -d '@' -f 2 | cut -d ':' -f 1 > $LIST #no ipv6 in this list
 is_file_empty "${LIST}"
+./mergeipaddrlist.py $LIST > $LIST_MERGED
+is_file_empty "${LIST_MERGED}"
 
-./ipaddr2list.py $LIST NDPI_PROTOCOL_MINING > $DEST
-rm -f $TMP $LIST
+./ipaddr2list.py $LIST_MERGED NDPI_PROTOCOL_ETHEREUM > $DEST
+rm -f $TMP $LIST $LIST_MERGED
 is_file_empty "${DEST}"
 
-echo "(3) Ethereum/Mining IPs are available in $DEST"
+echo "(3) Ethereum IPs are available in $DEST"
 exit 0
