@@ -30,6 +30,14 @@ RETVAL=$(( RETVAL + $? ))
 RETVAL=$(( RETVAL + $? ))
 ./crawlers_ip_addresses_download.sh
 RETVAL=$(( RETVAL + $? ))
+./nordvpn_ip_addresses_download.sh
+RETVAL=$(( RETVAL + $? ))
+./surfshark_ip_addresses_download.sh
+RETVAL=$(( RETVAL + $? ))
+./digitalocean_ip_addresses_download.sh
+RETVAL=$(( RETVAL + $? ))
+./tor_exit_nodes_ip_addresses_download.sh
+RETVAL=$(( RETVAL + $? ))
 
 ./asn_update.sh
 RETVAL=$(( RETVAL + $? ))
@@ -45,5 +53,8 @@ RETVAL=$(( RETVAL + $? ))
 ./public_suffix_list_download.sh
 RETVAL=$(( RETVAL + $? ))
 
+./microsoft_domains_download.sh
+RETVAL=$(( RETVAL + $? ))
+
 test ${RETVAL} -ne 0 && printf '%s: %s\n' "${0}" "${RETVAL} script(s) failed"
 exit ${RETVAL}
